/*
 * Jalview - A Sequence Alignment Editor and Viewer ($$Version-Rel$$)
 * Copyright (C) $$Year-Rel$$ The Jalview Authors
 * 
 * This file is part of Jalview.
 * 
 * Jalview is free software: you can redistribute it and/or
 * modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation, either version 3
 * of the License, or (at your option) any later version.
 *  
 * Jalview is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; without even the implied warranty 
 * of MERCHANTABILITY or FITNESS FOR A PARTICULAR 
 * PURPOSE.  See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Jalview.  If not, see <http://www.gnu.org/licenses/>.
 * The Jalview Authors are detailed in the 'AUTHORS' file.
 */
package jalview.util;

import java.awt.Toolkit;
import java.awt.event.MouseEvent;

import javax.swing.SwingUtilities;

/**
 * System platform information used by Applet and Application
 * 
 * @author Jim Procter
 */
public class Platform
{

  private static boolean isJS = /** @j2sNative true || */false;

  private static Boolean isNoJSMac = null, isNoJSWin = null, 
		  isMac = null, isWin = null;
  
  private static Boolean isHeadless = null;

  /**
   * added to group mouse events into Windows and nonWindows (mac, unix, linux)
   * @return
   */
  public static boolean isMac()
  {
	  return (isMac == null ? (isMac = (System.getProperty("os.name").indexOf("Mac") >= 0)) : isMac);
  }

  /**
   * added to group mouse events into Windows and nonWindows (mac, unix, linux)
   * @return
   */
  public static boolean isWin() 
  {
	  return (isWin == null ? (isWin = (System.getProperty("os.name").indexOf("Win") >= 0)) : isWin);
  }

  /**
   * 
   * @return true if HTML5 JavaScript
   */
  public static boolean isJS()
  {
	return isJS;
  }

  /**
   * sorry folks - Macs really are different
   * 
   * BH: disabled for SwingJS -- will need to check key-press issues
   * 
   * @return true if we do things in a special way.
   */
  public static boolean isAMacAndNotJS()
  {
	return (isNoJSMac == null ? (isNoJSMac = !isJS && isMac()) : isNoJSMac);
  }

/**
   * Check if we are on a Microsoft plaform...
   * 
   * @return true if we have to cope with another platform variation
   */
  public static boolean isWindowsAndNotJS()
  {
	return (isNoJSWin == null ? (isNoJSWin = !isJS && isWin()) : isNoJSWin);
   }

  /**
   * 
   * @return true if we are running in non-interactive no UI mode
   */
  public static boolean isHeadless()
  {
    if (isHeadless == null)
    {
      isHeadless = "true".equals(System.getProperty("java.awt.headless"));
    }
    return isHeadless;
  }

  /**
   * 
   * @return nominal maximum command line length for this platform
   */
  public static int getMaxCommandLineLength()
  {
    // TODO: determine nominal limits for most platforms.
    return 2046; // this is the max length for a windows NT system.
  }

  /**
   * escape a string according to the local platform's escape character
   * 
   * @param file
   * @return escaped file
   */
  public static String escapeString(String file)
  {
    StringBuffer f = new StringBuffer();
    int p = 0, lastp = 0;
    while ((p = file.indexOf('\\', lastp)) > -1)
    {
      f.append(file.subSequence(lastp, p));
      f.append("\\\\");
      lastp = p + 1;
    }
    f.append(file.substring(lastp));
    return f.toString();
  }

  /**
   * Answers true if the mouse event has Meta-down (Command key on Mac) or
   * Ctrl-down (on other o/s). Note this answers _false_ if the Ctrl key is
   * pressed instead of the Meta/Cmd key on Mac. To test for Ctrl-pressed on Mac,
   * you can use e.isPopupTrigger().
   * 
   * @param e
   * @return
   */
  public static boolean isControlDown(MouseEvent e)
  {
    return isControlDown(e, isMac());
  }

  /**
   * Overloaded version of method (to allow unit testing)
   * 
   * @param e
   * @param aMac
   * @return
   */
  protected static boolean isControlDown(MouseEvent e, boolean aMac)
  {
    if (!aMac) {
        return e.isControlDown();    	
    }
      // answer false for right mouse button
      // shortcut key will be META for a Mac 
    return !e.isPopupTrigger() 
    		  && (Toolkit.getDefaultToolkit().getMenuShortcutKeyMask() & e.getModifiers()) != 0;
      // could we use e.isMetaDown() here?
  }

  /**
   * Windows (not Mac, Linux, or Unix) and right button
   * to test for the right-mouse pressed event in Windows
   * that would have opened a menu or a Mac. 
   * 
   * @param e
   * @return
   */
  public static boolean isWinRightButton(MouseEvent e) 
  {
	  // was !isAMac(), but that is true also for Linux and Unix and JS, 

	  return isWin() && SwingUtilities.isRightMouseButton(e);
  }
  
  
  /**
   * Windows (not Mac, Linux, or Unix) and middle button -- for mouse wheeling 
   * without pressing the button.
   * 
   * @param e
   * @return
   */
  public static boolean isWinMiddleButton(MouseEvent e) 
  {
	// was !isAMac(), but that is true also for Linux and Unix and JS
	  return isWin() && SwingUtilities.isMiddleMouseButton(e);
  }

  public static boolean allowMnemonics() 
  {
	return !isMac();
  }
  
  public final static int TIME_RESET = 0;
  public final static int TIME_MARK  = 1;
  public static final int TIME_SET   = 2;
  public static final int TIME_GET   = 3;
  
  public static long time, mark, set, duration;
  
  public static void timeCheck(String msg, int mode) {
	  long t = System.currentTimeMillis();
	  switch (mode) {
	  case TIME_RESET:
		  time = mark = t;
		  if (msg != null)
			  System.err.println("Platform: timer reset\t\t\t" + msg);
		  break;
	  case TIME_MARK:
		  if (set > 0) {
			  duration += (t - set);
	  	  } else {
			  if (time == 0)
				  time = mark = t;
			  if (msg != null)
				  System.err.println("Platform: timer mark\t" + ((t - time)/1000f) + "\t" + ((t - mark)/1000f) + "\t" + msg);
			  mark = t;
	  	  }
		  break;
	  case TIME_SET:
		  set = t;
		  break;
	  case TIME_GET:
		  if (msg != null)
			  System.err.println("Platform: timer dur\t" + ((t - time)/1000f) + "\t" + ((duration)/1000f) + "\t" + msg);
		  set = 0;
		  break;
	  }
  }

   
}
